@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012-2014 Ben Croston
+Copyright (c) 2012-2015 Ben Croston
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
@@ -20,25 +20,26 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */
 
+#include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <sys/mman.h>
 #include "c_gpio.h"
 
-#define BCM2708_PERI_BASE   0x20000000
-#define GPIO_BASE           (BCM2708_PERI_BASE + 0x200000)
-#define FSEL_OFFSET         0   // 0x0000
-#define SET_OFFSET          7   // 0x001c / 4
-#define CLR_OFFSET          10  // 0x0028 / 4
-#define PINLEVEL_OFFSET     13  // 0x0034 / 4
-#define EVENT_DETECT_OFFSET 16  // 0x0040 / 4
-#define RISING_ED_OFFSET    19  // 0x004c / 4
-#define FALLING_ED_OFFSET   22  // 0x0058 / 4
-#define HIGH_DETECT_OFFSET  25  // 0x0064 / 4
-#define LOW_DETECT_OFFSET   28  // 0x0070 / 4
-#define PULLUPDN_OFFSET     37  // 0x0094 / 4
-#define PULLUPDNCLK_OFFSET  38  // 0x0098 / 4
+#define BCM2708_PERI_BASE_DEFAULT   0x20000000
+#define GPIO_BASE_OFFSET            0x200000
+#define FSEL_OFFSET                 0   // 0x0000
+#define SET_OFFSET                  7   // 0x001c / 4
+#define CLR_OFFSET                  10  // 0x0028 / 4
+#define PINLEVEL_OFFSET             13  // 0x0034 / 4
+#define EVENT_DETECT_OFFSET         16  // 0x0040 / 4
+#define RISING_ED_OFFSET            19  // 0x004c / 4
+#define FALLING_ED_OFFSET           22  // 0x0058 / 4
+#define HIGH_DETECT_OFFSET          25  // 0x0064 / 4
+#define LOW_DETECT_OFFSET           28  // 0x0070 / 4
+#define PULLUPDN_OFFSET             37  // 0x0094 / 4
+#define PULLUPDNCLK_OFFSET          38  // 0x0098 / 4
 
 #define PAGE_SIZE  (4*1024)
 #define BLOCK_SIZE (4*1024)
@@ -58,7 +59,23 @@ int setup(void)
 {
     int mem_fd;
     uint8_t *gpio_mem;
+    uint32_t peri_base = BCM2708_PERI_BASE_DEFAULT;
+    uint32_t gpio_base; 
+    unsigned char buf[4];
+    FILE *fp;
 
+    // get peri base from device tree
+    if ((fp = fopen("/proc/device-tree/soc/ranges", "rb")) != NULL) {
+        fseek(fp, 4, SEEK_SET);
+        if (fread(buf, 1, sizeof buf, fp) == sizeof buf) {
+            peri_base = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3] << 0;
+        }
+        fclose(fp);
+    }
+
+    gpio_base = peri_base + GPIO_BASE_OFFSET;
+
+    // mmap the GPIO memory registers
     if ((mem_fd = open("/dev/mem", O_RDWR|O_SYNC) ) < 0)
         return SETUP_DEVMEM_FAIL;
 
@@ -68,7 +85,7 @@ int setup(void)
     if ((uint32_t)gpio_mem % PAGE_SIZE)
         gpio_mem += PAGE_SIZE - ((uint32_t)gpio_mem % PAGE_SIZE);
 
-    gpio_map = (uint32_t *)mmap( (caddr_t)gpio_mem, BLOCK_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_FIXED, mem_fd, GPIO_BASE);
+    gpio_map = (uint32_t *)mmap( (caddr_t)gpio_mem, BLOCK_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_FIXED, mem_fd, gpio_base);
 
     if ((uint32_t)gpio_map < 0)
         return SETUP_MMAP_FAIL;
