@@ -29,7 +29,7 @@ void set_rising_event(int gpio, int enable);
 void set_falling_event(int gpio, int enable);
 void set_high_event(int gpio, int enable);
 void set_low_event(int gpio, int enable);
-int event_detected(int gpio);
+int eventdetected(int gpio);
 void cleanup(void);
 
 #define SETUP_OK          0
